#version 430 core
/*
* Copyright 2018 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

out vec4 color;

uniform vec3 camera_world;

uniform float time;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
//uniform sampler2D diffuseTexture;
uniform sampler2D normalMap; 
uniform bool normalmaping; 

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
};

#define NR_POINTL 4

uniform PointLight pointLs[NR_POINTL];



vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

float random (float x) {
    return fract(sin(x)*1e4);
}

float random(vec2 pos){
	return fract(sin(dot(pos.xy,vec2(12.9898,78.233)))*43758.5453123);
}

float randomSerie(float x, float freq, float t) {
    return step(.8,random( floor(x*freq)-floor(t) ));
}

void main() {	
		vec3 n = vec3(1.0);
	if (normalmaping) {
		n = texture(normalMap, vert.uv).rgb; 
		//n = normalize(n);
		if(vert.normal_world.y == 1.0) 
		{
			//inside = true; 
			vec3 temp = n; 
			n.x = temp.x; 
			n.y = temp.z; 
			n.z = temp.y * (1.0);
		}
		
		else if (vert.normal_world.r == 1.0)
		{
			vec3 temp = n; 
			n.x = temp.z; 
			n.y = temp.y; 
			n.z = temp.x * (-1.0); 
		}  
		else if (vert.normal_world.r == (-1.0))
		{
			vec3 temp = n; 
			n.x = temp.z * (-1.0); 
			n.z = temp.x * (-1.0); 
		
		}
		else if (vert.normal_world.z == 1.0) {
			//inside = true; 
			// solten die normalen normal map cordinaten sein 
		} else if (vert.normal_world.z == -1.0) {
			//inside = true; 
			vec3 temp = n; 
			n.x = temp.x * (-1.0); 
			n.z = temp.z * (-1.0); 
		}

		n = normalize(n); 
	} 
	else 
	{
		n = normalize(vert.normal_world);
	}
	vec3 v = normalize(camera_world - vert.position_world);

	vec2 pos = vert.uv;

	float cols = 30.0;
	float freq = random(floor(time))+abs(atan(time)*0.1);

    float t = 60.+time*(1.0-freq)*30.;

    if (fract(pos.y*cols* 0.5) < 0.5){
        t *= -1.0;
    }

    freq += random(floor(pos.y));

	float rand = random(floor(pos*10));
	float offset = 0.025;

	vec3 texColor = vec3(randomSerie(pos.x, freq*100.0, t-offset), randomSerie(pos.x, freq*100., t), randomSerie(pos.x, freq*100., t-offset));

	color = vec4(texColor * materialCoefficients.x, 1); // ambient


	// add directional light contribution
	color.rgb += phong(n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));
			
	// add point lights contribution
	for(int j=0; j<NR_POINTL; j++){
		color.rgb += phong(n,  pointLs[j].position - vert.position_world, v, pointLs[j].color * texColor, materialCoefficients.y, pointLs[j].color, materialCoefficients.z, specularAlpha, true, pointLs[j].attenuation);
	}
}

