#version 430 core
/*
* Copyright 2018 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

out vec4 color;

uniform vec3 camera_world;

uniform float time;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
//uniform sampler2D diffuseTexture;
uniform sampler2D normalMap; 
uniform bool normalmaping; 

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
};

#define NR_POINTL 4

uniform PointLight pointLs[NR_POINTL];



vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

float random (in float x) {
    return fract(sin(x)*1e4);
}

float random(vec2 pos){
	return fract(sin(dot(pos.xy,vec2(12.9898,78.233)))*43758.5453123);
}

float noise(vec2 pos){
		vec2 x = floor(pos);
		vec2 y = fract(pos);

		float a,b,c,d;

		a = random(x);
		b = random(x + vec2(1.0, 0.0));
		c = random(x + vec2(0.0, 1.0));
		d = random(x + vec2(1.0, 1.0));


		vec2 e =  y*y*(3.0-2.0*y);

		return mix(a, b, e.x) + (c - a)* e.y * (1.0 - e.x) + (d - b) * e.x * e.y;

}

void main() {	
		vec3 n = vec3(1.0);
	if (normalmaping) {
		n = texture(normalMap, vert.uv).rgb; 
		//n = normalize(n);
		if(vert.normal_world.y == 1.0) 
		{
			//inside = true; 
			vec3 temp = n; 
			n.x = temp.x; 
			n.y = temp.z; 
			n.z = temp.y * (1.0);
		}
		
		else if (vert.normal_world.r == 1.0)
		{
			vec3 temp = n; 
			n.x = temp.z; 
			n.y = temp.y; 
			n.z = temp.x * (-1.0); 
		}  
		else if (vert.normal_world.r == (-1.0))
		{
			vec3 temp = n; 
			n.x = temp.z * (-1.0); 
			n.z = temp.x * (-1.0); 
		
		}
		else if (vert.normal_world.z == 1.0) {
			//inside = true; 
			// solten die normalen normal map cordinaten sein 
		} else if (vert.normal_world.z == -1.0) {
			//inside = true; 
			vec3 temp = n; 
			n.x = temp.x * (-1.0); 
			n.z = temp.z * (-1.0); 
		}

		n = normalize(n); 
	} 
	else 
	{
		n = normalize(vert.normal_world);
	}
	vec3 v = normalize(camera_world - vert.position_world);

	vec2 pos = vert.uv;



	//parameters 2 play
	float cloudiness =7.0;
	vec2 offset = vec2(time/10,time/10);
	vec3 texColor = vec3(noise((pos+offset)*cloudiness*1.5));
	//vec3 texColor = vec3(0.5, 0.5, 0.5);

	color = vec4(texColor * materialCoefficients.x, 1); // ambient

	// add directional light contribution
	color.rgb += phong(n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));
			
	// add point lights contribution
	for(int j=0; j<NR_POINTL; j++){
		color.rgb += phong(n,  pointLs[j].position - vert.position_world, v, pointLs[j].color * texColor, materialCoefficients.y, pointLs[j].color, materialCoefficients.z, specularAlpha, true, pointLs[j].attenuation);
	}
}

